package com.zanxiang.game.back.serve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.back.serve.dao.mapper.GameTencentAppCallbackMapper;
import com.zanxiang.game.back.serve.pojo.dto.GameTencentAppCallbackDTO;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentAppCallback;
import com.zanxiang.game.back.serve.service.IGameTencentAppCallbackService;
import com.zanxiang.module.util.DateUtil;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.bean.BeanUtil;
import com.zanxiang.module.util.encryption.Md5Util;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;

@Slf4j
@Service
public class GameTencentAppCallbackServiceImpl extends ServiceImpl<GameTencentAppCallbackMapper, GameTencentAppCallback>
implements IGameTencentAppCallbackService {

    @Value("${spring.kafka.tencentAppCallbackTopic}")
    private String tencentAppCallbackTopic;
    @Autowired
    private KafkaProducer<String, String> kafkaProducer;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean callback(GameTencentAppCallbackDTO dto) {
        GameTencentAppCallback appCallback = BeanUtil.copy(dto, GameTencentAppCallback.class);
        appCallback.setDay(dto.getClickTime() == null || dto.getClickTime() < 1000 ? LocalDate.now() : DateUtil.milliToLocalDateTime(dto.getClickTime()).toLocalDate());
        save(appCallback);
        try {
            kafkaProducer.send(new ProducerRecord<>(tencentAppCallbackTopic, dto.getAccountId().toString(), JsonUtil.toString(appCallback)));
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return true;
    }

    public GameTencentAppCallback getUserClickInfo(String imei, String oaid, String android, String idfa, String caid) {
        LambdaQueryWrapper<GameTencentAppCallback> qw = new LambdaQueryWrapper<>();
        if (StringUtils.isNoneBlank(imei)) {
            qw.or().eq(GameTencentAppCallback::getMuid, Md5Util.encrypt32(imei.toLowerCase()).toLowerCase());
        } else if (StringUtils.isNoneBlank(idfa)) {
            qw.or().eq(GameTencentAppCallback::getMuid, Md5Util.encrypt32(idfa).toLowerCase());
        }
        if(StringUtils.isNoneBlank(android)) {
            qw.or().eq(GameTencentAppCallback::getHashAndroidId, Md5Util.encrypt32(android).toLowerCase());
        }
        if(StringUtils.isNoneBlank(oaid)) {
            qw.or().eq(GameTencentAppCallback::getHashOaid, Md5Util.encrypt32(oaid).toLowerCase());
        }
        qw.orderByDesc(GameTencentAppCallback::getClickTime)
                .last("limit 1");
        return getOne(qw);
    }
}
